@@ -43,11 +43,10 @@
 #include <gio/gio.h>
 
 #include "gedit-utils.h"
-#include "gedit-convert.h"
 #include "gedit-document.h"
 #include "gedit-io-error-message-area.h"
 #include "gedit-prefs-manager.h"
-#include <gedit/gedit-encodings-option-menu.h>
+#include <gedit/gedit-encodings-combo-box.h>
 
 #if !GTK_CHECK_VERSION (2, 17, 1)
 #include "gedit-message-area.h"
@@ -170,7 +169,8 @@ set_message_area_text_and_icon (GtkWidget   *message_area,
 
 static GtkWidget *
 create_io_loading_error_message_area (const gchar *primary_text,
-				      const gchar *secondary_text)
+				      const gchar *secondary_text,
+				      gboolean     recoverable_error)
 {
 	GtkWidget *message_area;
 
@@ -191,6 +191,21 @@ create_io_loading_error_message_area (const gchar *primary_text,
 					primary_text,
 					secondary_text);
 
+	if (recoverable_error)
+	{
+#if !GTK_CHECK_VERSION (2, 17, 1)
+		gedit_message_area_add_stock_button_with_text (GEDIT_MESSAGE_AREA (message_area),
+							       _("_Retry"),
+							       GTK_STOCK_REFRESH,
+							       GTK_RESPONSE_OK);
+#else
+		info_bar_add_stock_button_with_text (GTK_INFO_BAR (message_area),
+						     _("_Retry"),
+						     GTK_STOCK_REFRESH,
+						     GTK_RESPONSE_OK);
+#endif
+	}
+
 	return message_area;
 }
 
@@ -377,77 +392,6 @@ parse_error (const GError *error,
 	}
 }
 
-GtkWidget *
-gedit_io_loading_error_message_area_new (const gchar  *uri,
-					 const GError *error)
-{
-	gchar *error_message = NULL;
-	gchar *message_details = NULL;
-	gchar *full_formatted_uri;
-	gchar *uri_for_display;
-	gchar *temp_uri_for_display;
-	GtkWidget *message_area;
-
-	g_return_val_if_fail (uri != NULL, NULL);
-	g_return_val_if_fail (error != NULL, NULL);
-	g_return_val_if_fail ((error->domain == GEDIT_DOCUMENT_ERROR) || 
-			      (error->domain == G_IO_ERROR), NULL);
-
-	full_formatted_uri = gedit_utils_uri_for_display (uri);
-
-	/* Truncate the URI so it doesn't get insanely wide. Note that even
-	 * though the dialog uses wrapped text, if the URI doesn't contain
-	 * white space then the text-wrapping code is too stupid to wrap it.
-	 */
-	temp_uri_for_display = gedit_utils_str_middle_truncate (full_formatted_uri, 
-								MAX_URI_IN_DIALOG_LENGTH);								
-	g_free (full_formatted_uri);
-
-	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
-	g_free (temp_uri_for_display);
-
-	if (is_gio_error (error, G_IO_ERROR_TOO_MANY_LINKS))
-	{
-		message_details = g_strdup (_("The number of followed links is limited and the actual file could not be found within this limit."));
-	}
-	else if (is_gio_error (error, G_IO_ERROR_PERMISSION_DENIED))
-	{
-		message_details = g_strdup (_("You do not have the permissions necessary to open the file."));
-	}
-	else
-	{
-		parse_error (error, &error_message, &message_details, uri, uri_for_display);
-	}
-
-	if (error_message == NULL)
-		error_message = g_strdup_printf (_("Could not open the file %s."),
-						 uri_for_display);
-
-	message_area = create_io_loading_error_message_area (error_message,
-							     message_details);
-
-	if (is_recoverable_error (error))
-	{
-#if !GTK_CHECK_VERSION (2, 17, 1)
-		gedit_message_area_add_stock_button_with_text (GEDIT_MESSAGE_AREA (message_area),
-							       _("_Retry"),
-							       GTK_STOCK_REFRESH,
-							       GTK_RESPONSE_OK);
-#else
-		info_bar_add_stock_button_with_text (GTK_INFO_BAR (message_area),
-						     _("_Retry"),
-						     GTK_STOCK_REFRESH,
-						     GTK_RESPONSE_OK);
-#endif
-	}
-
-	g_free (uri_for_display);
-	g_free (error_message);
-	g_free (message_details);
-
-	return message_area;
-}
-
 GtkWidget *
 gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 						      const GError *error)
@@ -492,7 +436,8 @@ gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 						 uri_for_display);
 
 	message_area = create_io_loading_error_message_area (error_message,
-							     message_details);
+							     message_details,
+							     FALSE);
 
 	g_free (uri_for_display);
 	g_free (error_message);
@@ -502,7 +447,7 @@ gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 }
 
 static void
-create_option_menu (GtkWidget *message_area, GtkWidget *vbox)
+create_combo_box (GtkWidget *message_area, GtkWidget *vbox)
 {
 	GtkWidget *hbox;
 	GtkWidget *label;
@@ -516,7 +461,7 @@ create_option_menu (GtkWidget *message_area, GtkWidget *vbox)
 	label = gtk_label_new_with_mnemonic (label_markup);
 	g_free (label_markup);
 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	menu = gedit_encodings_option_menu_new (TRUE);
+	menu = gedit_encodings_combo_box_new (TRUE);
 	g_object_set_data (G_OBJECT (message_area), 
 			   "gedit-message-area-encoding-menu", 
 			   menu);
@@ -540,7 +485,8 @@ create_option_menu (GtkWidget *message_area, GtkWidget *vbox)
 
 static GtkWidget *
 create_conversion_error_message_area (const gchar *primary_text,
-				      const gchar *secondary_text)
+				      const gchar *secondary_text,
+				      gboolean     edit_anyway)
 {
 	GtkWidget *message_area;
 	GtkWidget *hbox_content;
@@ -558,21 +504,53 @@ create_conversion_error_message_area (const gchar *primary_text,
 						       _("_Retry"),
 						       GTK_STOCK_REDO,
 						       GTK_RESPONSE_OK);
-	gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
-				       GTK_STOCK_CANCEL,
-				       GTK_RESPONSE_CANCEL);
+
+	if (edit_anyway)
+	{
+		gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
+					       _("Edit Any_way"),
+					       GTK_RESPONSE_YES);
+		gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
+					       _("D_on't Edit"),
+					       GTK_RESPONSE_NO);
+	}
+	else
+	{
+		gedit_message_area_add_button (GEDIT_MESSAGE_AREA (message_area),
+					       GTK_STOCK_CANCEL,
+					       GTK_RESPONSE_CANCEL);
+	}
 #else
 	message_area = gtk_info_bar_new ();
+
 	info_bar_add_stock_button_with_text (GTK_INFO_BAR (message_area),
 					     _("_Retry"),
 					     GTK_STOCK_REDO,
 					     GTK_RESPONSE_OK);
 
-	gtk_info_bar_add_button (GTK_INFO_BAR (message_area),
-				 GTK_STOCK_CANCEL,
-				 GTK_RESPONSE_CANCEL);
-	gtk_info_bar_set_message_type (GTK_INFO_BAR (message_area),
-				       GTK_MESSAGE_ERROR);
+	if (edit_anyway)
+	{
+		gtk_info_bar_add_button (GTK_INFO_BAR (message_area),
+		/* Translators: the access key chosen for this string should be
+		 different from other main menu access keys (Open, Edit, View...) */
+					 _("Edit Any_way"),
+					 GTK_RESPONSE_YES);
+		gtk_info_bar_add_button (GTK_INFO_BAR (message_area),
+		/* Translators: the access key chosen for this string should be
+		 different from other main menu access keys (Open, Edit, View...) */
+					 _("D_on't Edit"),
+					 GTK_RESPONSE_NO);
+		gtk_info_bar_set_message_type (GTK_INFO_BAR (message_area),
+					       GTK_MESSAGE_WARNING);
+	}
+	else
+	{
+		gtk_info_bar_add_button (GTK_INFO_BAR (message_area),
+					 GTK_STOCK_CANCEL,
+					 GTK_RESPONSE_CANCEL);
+		gtk_info_bar_set_message_type (GTK_INFO_BAR (message_area),
+					       GTK_MESSAGE_ERROR);
+	}
 #endif
 
 	hbox_content = gtk_hbox_new (FALSE, 8);
@@ -608,7 +586,7 @@ create_conversion_error_message_area (const gchar *primary_text,
 		gtk_misc_set_alignment (GTK_MISC (secondary_label), 0, 0.5);
 	}
 
-	create_option_menu (message_area, vbox);
+	create_combo_box (message_area, vbox);
 	gtk_widget_show_all (hbox_content);
 	set_contents (message_area, hbox_content);
 
@@ -616,10 +594,9 @@ create_conversion_error_message_area (const gchar *primary_text,
 }
 
 GtkWidget *
-gedit_conversion_error_while_loading_message_area_new (
-						const gchar         *uri,
-						const GeditEncoding *encoding,
-				    		const GError        *error)
+gedit_io_loading_error_message_area_new (const gchar         *uri,
+					 const GeditEncoding *encoding,
+					 const GError        *error)
 {
 	gchar *error_message = NULL;
 	gchar *message_details = NULL;
@@ -628,11 +605,14 @@ gedit_conversion_error_while_loading_message_area_new (
 	gchar *uri_for_display;
 	gchar *temp_uri_for_display;
 	GtkWidget *message_area;
+	gboolean edit_anyway = FALSE;
+	gboolean convert_error = FALSE;
 	
 	g_return_val_if_fail (uri != NULL, NULL);
 	g_return_val_if_fail (error != NULL, NULL);
 	g_return_val_if_fail ((error->domain == G_CONVERT_ERROR) ||
-			      (error->domain == GEDIT_CONVERT_ERROR), NULL);
+			      (error->domain == GEDIT_DOCUMENT_ERROR) ||
+			      (error->domain == G_IO_ERROR), NULL);
 	
 	full_formatted_uri = gedit_utils_uri_for_display (uri);
 
@@ -640,8 +620,8 @@ gedit_conversion_error_while_loading_message_area_new (
 	 * though the dialog uses wrapped text, if the URI doesn't contain
 	 * white space then the text-wrapping code is too stupid to wrap it.
 	 */
-	temp_uri_for_display = gedit_utils_str_middle_truncate (full_formatted_uri, 
-								MAX_URI_IN_DIALOG_LENGTH);								
+	temp_uri_for_display = gedit_utils_str_middle_truncate (full_formatted_uri,
+								MAX_URI_IN_DIALOG_LENGTH);
 	g_free (full_formatted_uri);
 	
 	uri_for_display = g_markup_printf_escaped ("<i>%s</i>", temp_uri_for_display);
@@ -652,28 +632,65 @@ gedit_conversion_error_while_loading_message_area_new (
 	else
 		encoding_name = g_strdup ("UTF-8");
 
-	if (error->domain == GEDIT_CONVERT_ERROR)
+	if (is_gio_error (error, G_IO_ERROR_TOO_MANY_LINKS))
+	{
+		message_details = g_strdup (_("The number of followed links is limited and the actual file could not be found within this limit."));
+	}
+	else if (is_gio_error (error, G_IO_ERROR_PERMISSION_DENIED))
+	{
+		message_details = g_strdup (_("You do not have the permissions necessary to open the file."));
+	}
+	else if ((is_gio_error (error, G_IO_ERROR_INVALID_DATA) && encoding == NULL) ||
+	         (error->domain == GEDIT_DOCUMENT_ERROR &&
+	         error->code == GEDIT_DOCUMENT_ERROR_ENCODING_AUTO_DETECTION_FAILED))
 	{
-		g_return_val_if_fail (error->code == GEDIT_CONVERT_ERROR_AUTO_DETECTION_FAILED, NULL);
-		
 		error_message = g_strdup_printf (_("Could not open the file %s."),
-							 uri_for_display);
+						 uri_for_display);
 		message_details = g_strconcat (_("gedit has not been able to detect "
 				               "the character coding."), "\n", 
 				               _("Please check that you are not trying to open a binary file."), "\n",
 					       _("Select a character coding from the menu and try again."), NULL);
+		convert_error = TRUE;
 	}
-	else 
+	else if (error->domain == GEDIT_DOCUMENT_ERROR &&
+	         error->code == GEDIT_DOCUMENT_ERROR_CONVERSION_FALLBACK)
+	{
+		error_message = g_strdup_printf (_("There was a problem opening the file %s."),
+						 uri_for_display);
+		message_details = g_strconcat (_("The file you opened has some invalid characters, "
+					       "if you continue editing this file you could make this "
+					       "document useless."), "\n",
+					       _("You can also choose another character encoding and try again."),
+					       NULL);
+		edit_anyway = TRUE;
+		convert_error = TRUE;
+	}
+	else if (is_gio_error (error, G_IO_ERROR_INVALID_DATA) && encoding != NULL)
 	{
-		
 		error_message = g_strdup_printf (_("Could not open the file %s using the %s character coding."),
 						 uri_for_display, 
 						 encoding_name);
 		message_details = g_strconcat (_("Please check that you are not trying to open a binary file."), "\n",
 					       _("Select a different character coding from the menu and try again."), NULL);
+		convert_error = TRUE;
+	}
+	else
+	{
+		parse_error (error, &error_message, &message_details, uri, uri_for_display);
+	}
+
+	if (convert_error)
+	{
+		message_area = create_conversion_error_message_area (error_message,
+								     message_details,
+								     edit_anyway);
+	}
+	else
+	{
+		message_area = create_io_loading_error_message_area (error_message,
+								     message_details,
+								     is_recoverable_error (error));
 	}
-	
-	message_area = create_conversion_error_message_area (error_message, message_details);
 
 	g_free (uri_for_display);
 	g_free (encoding_name);
@@ -726,7 +743,8 @@ gedit_conversion_error_while_saving_message_area_new (
 	
 	message_area = create_conversion_error_message_area (
 								error_message,
-								message_details);
+								message_details,
+								FALSE);
 
 	g_free (uri_for_display);
 	g_free (encoding_name);
@@ -751,8 +769,8 @@ gedit_conversion_error_message_area_get_encoding (GtkWidget *message_area)
 				  "gedit-message-area-encoding-menu");	
 	g_return_val_if_fail (menu, NULL);
 	
-	return gedit_encodings_option_menu_get_selected_encoding
-					(GEDIT_ENCODINGS_OPTION_MENU (menu));
+	return gedit_encodings_combo_box_get_selected_encoding
+					(GEDIT_ENCODINGS_COMBO_BOX (menu));
 }
 
 GtkWidget *
@@ -1179,7 +1197,8 @@ gedit_unrecoverable_saving_error_message_area_new (const gchar  *uri,
 						 uri_for_display);
 
 	message_area = create_io_loading_error_message_area (error_message,
-							     message_details);
+							     message_details,
+							     FALSE);
 
 	g_free (uri_for_display);
 	g_free (error_message);
