@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-panel.c 6472 2008-08-29 13:17:31Z icq $
+ * $Id$
  */
 
 #include "gedit-panel.h"
@@ -35,6 +35,7 @@
 #include <gdk/gdk.h>
 #include <gdk/gdkkeysyms.h>
 
+#include "gedit-close-button.h"
 #include "gedit-window.h"
 #include "gedit-debug.h"
 
@@ -79,15 +80,6 @@ enum {
 
 static guint signals[LAST_SIGNAL];
 
-static void	 gedit_panel_set_property	(GObject         *object,
-						 guint            prop_id,
-						 const GValue    *value,
-						 GParamSpec      *pspec);
-
-static void	 gedit_panel_class_init		(GeditPanelClass *klass);
-
-static void	 gedit_panel_init 		(GeditPanel *panel);
-
 static GObject	*gedit_panel_constructor	(GType type,
 						 guint n_construct_properties,
 						 GObjectConstructParam *construct_properties);
@@ -248,7 +240,7 @@ gedit_panel_class_init (GeditPanelClass *klass)
 		  	      g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);					
 	binding_set = gtk_binding_set_by_class (klass);
-  
+
 	gtk_binding_entry_add_signal (binding_set, 
 				      GDK_Escape, 
 				      0, 
@@ -403,6 +395,12 @@ panel_show (GeditPanel *panel,
 		notebook_page_changed (nb, NULL, page, panel);
 }
 
+static void
+gedit_panel_init (GeditPanel *panel)
+{
+	panel->priv = GEDIT_PANEL_GET_PRIVATE (panel);
+}
+
 static void
 close_button_clicked_cb (GtkWidget *widget,
 			 GtkWidget *panel)
@@ -410,51 +408,19 @@ close_button_clicked_cb (GtkWidget *widget,
 	gtk_widget_hide (panel);
 }
 
-static void
-gedit_panel_init (GeditPanel *panel)
-{
-	panel->priv = GEDIT_PANEL_GET_PRIVATE (panel);
-	g_return_if_fail (panel->priv != NULL);	
-}
-
-static void
-button_style_set_cb (GtkWidget *button,
-		     GtkStyle *previous_style,
-		     gpointer user_data)
-{
-	gint h, w;
-
-	gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (button),
-					   GTK_ICON_SIZE_MENU, &w, &h);
-
-	gtk_widget_set_size_request (button, w + 2, h + 2);
-}
-
 static GtkWidget *
-create_small_button (GtkWidget *image)
+create_close_button (GeditPanel *panel)
 {
 	GtkWidget *button;
-	GtkRcStyle *rcstyle;
-	
-	button = gtk_button_new ();
-	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 
-	/* don't allow focus on the close button */
-	gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);
+	button = gedit_close_button_new ();
 
-	/* make it as small as possible */
-	rcstyle = gtk_rc_style_new ();
-	rcstyle->xthickness = rcstyle->ythickness = 0;
-	gtk_widget_modify_style (button, rcstyle);
-	gtk_rc_style_unref (rcstyle);
+	gtk_widget_set_tooltip_text (button, _("Hide panel"));
 
-	gtk_widget_show (image);
-
-	gtk_container_add (GTK_CONTAINER (button), image);
-
-	/* Set minimal size */
-	g_signal_connect (button, "style-set",
-			  G_CALLBACK (button_style_set_cb), NULL);
+	g_signal_connect (button,
+			  "clicked",
+			  G_CALLBACK (close_button_clicked_cb),
+			  panel);
 
 	return button;
 }
@@ -467,16 +433,16 @@ build_notebook_for_panel (GeditPanel *panel)
 
 	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (panel->priv->notebook),
 				  GTK_POS_BOTTOM);
-  	gtk_notebook_set_scrollable (GTK_NOTEBOOK (panel->priv->notebook),
-  				     TRUE);
-  	gtk_notebook_popup_enable (GTK_NOTEBOOK (panel->priv->notebook));
-  
-  	gtk_widget_show (GTK_WIDGET (panel->priv->notebook));
-  
-  	g_signal_connect (panel->priv->notebook,
-  			  "switch-page",
-  			  G_CALLBACK (notebook_page_changed),
-  			  panel);  
+	gtk_notebook_set_scrollable (GTK_NOTEBOOK (panel->priv->notebook),
+				     TRUE);
+	gtk_notebook_popup_enable (GTK_NOTEBOOK (panel->priv->notebook));
+
+	gtk_widget_show (GTK_WIDGET (panel->priv->notebook));
+
+	g_signal_connect (panel->priv->notebook,
+			  "switch-page",
+			  G_CALLBACK (notebook_page_changed),
+			  panel);
 }
 
 static void
@@ -485,8 +451,7 @@ build_horizontal_panel (GeditPanel *panel)
 	GtkWidget *box;
 	GtkWidget *sidebar;
 	GtkWidget *close_button;
-	GtkWidget *image;
-	
+
 	box = gtk_hbox_new(FALSE, 0);
 
 	gtk_box_pack_start (GTK_BOX (box), 
@@ -505,8 +470,7 @@ build_horizontal_panel (GeditPanel *panel)
 			    FALSE, 
 			    0);
 
-	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
-	close_button = create_small_button (image);
+	close_button = create_close_button (panel);
 
 	gtk_box_pack_start (GTK_BOX (sidebar),
 			    close_button,
@@ -514,13 +478,6 @@ build_horizontal_panel (GeditPanel *panel)
 			    FALSE, 
 			    0);
 
-	gtk_widget_set_tooltip_text (close_button, _("Hide panel"));
-
-	g_signal_connect (close_button,
-			  "clicked",
-                          G_CALLBACK (close_button_clicked_cb),
-                          panel);
-
 	gtk_widget_show_all (box);
 
 	gtk_box_pack_start (GTK_BOX (panel),
@@ -536,9 +493,8 @@ build_vertical_panel (GeditPanel *panel)
 	GtkWidget *close_button;
 	GtkWidget *title_hbox;
 	GtkWidget *icon_name_hbox;
-	GtkWidget *image;
 	GtkWidget *dummy_label;
-	
+
 	/* Create title hbox */
 	title_hbox = gtk_hbox_new (FALSE, 6);
 	gtk_container_set_border_width (GTK_CONTAINER (title_hbox), 5);
@@ -554,7 +510,7 @@ build_vertical_panel (GeditPanel *panel)
 		
 	panel->priv->title_image = 
 				gtk_image_new_from_stock (GTK_STOCK_FILE,
-							  GTK_ICON_SIZE_MENU); 							     
+							  GTK_ICON_SIZE_MENU);
 	gtk_box_pack_start (GTK_BOX (icon_name_hbox), 
 			    panel->priv->title_image, 
 			    FALSE, 
@@ -562,13 +518,13 @@ build_vertical_panel (GeditPanel *panel)
 			    0);
 
 	dummy_label = gtk_label_new (" ");
-		
+
 	gtk_box_pack_start (GTK_BOX (icon_name_hbox), 
 			    dummy_label, 
 			    FALSE, 
 			    FALSE, 
 			    0);	
-				    
+
 	panel->priv->title_label = gtk_label_new (_("Empty"));
 	gtk_misc_set_alignment (GTK_MISC (panel->priv->title_label), 0, 0.5);
 	gtk_label_set_ellipsize(GTK_LABEL (panel->priv->title_label), PANGO_ELLIPSIZE_END);
@@ -579,22 +535,14 @@ build_vertical_panel (GeditPanel *panel)
 			    TRUE,
 			    0);
 
-	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
-	close_button = create_small_button (image);
-				       
+	close_button = create_close_button (panel);
+
 	gtk_box_pack_start (GTK_BOX (title_hbox),
 			    close_button, 
 			    FALSE, 
 			    FALSE, 
 			    0);
 
-	gtk_widget_set_tooltip_text (close_button, _("Hide panel"));
-
-	g_signal_connect (close_button,
-			  "clicked",
-                          G_CALLBACK (close_button_clicked_cb),
-                          panel);
-
 	gtk_widget_show_all (title_hbox);
 
 	gtk_box_pack_start (GTK_BOX (panel),
@@ -689,7 +637,6 @@ build_tab_label (GeditPanel  *panel,
 
 	g_object_set_data (G_OBJECT (item), "label", label);
 	g_object_set_data (G_OBJECT (item), "hbox", hbox);
-	g_object_set_data (G_OBJECT (item), "label-ebox", label_ebox);
 
 	return hbox;
 }
@@ -799,7 +746,6 @@ gedit_panel_remove_item (GeditPanel *panel,
 {
 	GeditPanelItem *data;
 	gint page_num;
-	GtkWidget *ebox;
 	
 	g_return_val_if_fail (GEDIT_IS_PANEL (panel), FALSE);
 	g_return_val_if_fail (GTK_IS_WIDGET (item), FALSE);
@@ -821,8 +767,6 @@ gedit_panel_remove_item (GeditPanel *panel,
 		           PANEL_ITEM_KEY,
 		           NULL);
 
-	ebox = g_object_get_data (G_OBJECT (item), "label-ebox");
-	
 	/* ref the item to keep it alive during signal emission */
 	g_object_ref (G_OBJECT (item));
 
