@@ -144,9 +144,9 @@ void		 _gedit_tab_revert		(GeditTab            *tab);
 void		 _gedit_tab_save		(GeditTab            *tab);
 void		 _gedit_tab_save_as		(GeditTab            *tab,
 						 const gchar         *uri,
-						 const GeditEncoding *encoding);
+						 const GeditEncoding *encoding,
+						 GeditDocumentNewlineType newline_type);
 
-void		 _gedit_tab_page_setup		(GeditTab            *tab);
 void		 _gedit_tab_print		(GeditTab            *tab);
 void		 _gedit_tab_print_preview	(GeditTab            *tab);
 
@@ -154,6 +154,10 @@ void		 _gedit_tab_mark_for_closing	(GeditTab	     *tab);
 
 gboolean	 _gedit_tab_can_close		(GeditTab	     *tab);
 
+#if !GTK_CHECK_VERSION (2, 17, 4)
+void		 _gedit_tab_page_setup		(GeditTab            *tab);
+#endif
+
 G_END_DECLS
 
 #endif  /* __GEDIT_TAB_H__  */
