@@ -50,7 +50,6 @@
 #include "gedit-debug.h"
 #include "gedit-dirs.h"
 #include "gedit-encodings.h"
-#include "gedit-metadata-manager.h"
 #include "gedit-plugins-engine.h"
 #include "gedit-prefs-manager-app.h"
 #include "gedit-session.h"
@@ -67,12 +66,14 @@
 #include <conio.h>
 #define _WIN32_WINNT 0x0500
 #include <windows.h>
+#include "gedit-metadata-manager.h"
 #define DATADIR SAVE_DATADIR
 #undef SAVE_DATADIR
 #endif
 
 #ifdef OS_OSX
 #include <ige-mac-dock.h>
+#include <ige-mac-integration.h>
 #include "osx/gedit-osx.h"
 #endif
 
@@ -692,6 +693,10 @@ main (int argc, char *argv[])
 	gedit_debug_message (DEBUG_APP, "Init session manager");		
 	gedit_session_init ();
 
+#ifdef OS_OSX
+	ige_mac_menu_set_global_key_handler_enabled (FALSE);
+#endif
+
 	if (gedit_session_is_restored ())
 		restored = gedit_session_load ();
 
@@ -747,7 +752,10 @@ main (int argc, char *argv[])
 	 */
 	g_object_unref (engine);
 	gedit_prefs_manager_app_shutdown ();
+
+#ifdef G_OS_WIN32
 	gedit_metadata_manager_shutdown ();
+#endif
 
 	return 0;
 }
