@@ -18,10 +18,12 @@
 import os
 import weakref
 import sys
+import tempfile
+import re
 
 import gtk
 
-import ElementTree as et
+import xml.etree.ElementTree as et
 from Helper import *
 
 class NamespacedId:
@@ -38,7 +40,7 @@ class NamespacedId:
 
 class SnippetData:
         PROPS = {'tag': '', 'text': '', 'description': 'New snippet', 
-                        'accelerator': ''}
+                        'accelerator': '', 'drop-targets': ''}
 
         def __init__(self, node, library):
                 self.priv_id = node.attrib.get('id')
@@ -74,21 +76,21 @@ class SnippetData:
 
                 self.properties = {}
                 props = SnippetData.PROPS.copy()
-                
+
                 # Store all properties present
                 for child in node:
-                        if props.has_key(child.tag):
+                        if child.tag in props:
                                 del props[child.tag]
-                                
+
                                 # Normalize accelerator
                                 if child.tag == 'accelerator' and child.text != None:
                                         keyval, mod = gtk.accelerator_parse(child.text)
-                                        
-                                        if not gtk.accelerator_valid(keyval, mod):
-                                                child.text = ''
-                                        else:
+
+                                        if gtk.accelerator_valid(keyval, mod):
                                                 child.text = gtk.accelerator_name(keyval, mod)
-                                
+                                        else:
+                                                child.text = ''
+
                                 if self.can_modify():
                                         self.properties[child.tag] = child
                                 else:
@@ -107,7 +109,7 @@ class SnippetData:
                 self.check_validation()
         
         def check_validation(self):
-                if not self['tag'] and not self['accelerator']:
+                if not self['tag'] and not self['accelerator'] and not self['drop-targets']:
                         return False
 
                 library = Library()
@@ -116,19 +118,28 @@ class SnippetData:
                 self.valid = library.valid_tab_trigger(self['tag']) and \
                                 (not self['accelerator'] or library.valid_accelerator(keyval, mod))
         
+        def _format_prop(self, prop, value):
+                if prop == 'drop-targets' and value != '':
+                        return re.split('\\s*[,;]\\s*', value)
+                else:
+                        return value
+        
         def __getitem__(self, prop):
                 if prop in self.properties:
                         if self.can_modify():
-                                return self.properties[prop].text or ''
+                                return self._format_prop(prop, self.properties[prop].text or '')
                         else:
-                                return self.properties[prop] or ''
+                                return self._format_prop(prop, self.properties[prop] or '')
                 
-                return ''
+                return self._format_prop(prop, '')
         
         def __setitem__(self, prop, value):
                 if not prop in self.properties:
                         return
-                        
+                
+                if isinstance(value, list):
+                        value = ','.join(value)
+                               
                 if not self.can_modify() and self.properties[prop] != value:
                         # ohoh, this is not can_modify, but it needs to be changed...
                         # make sure it is transfered to the changes file and set all the
@@ -136,7 +147,6 @@ class SnippetData:
                         # This snippet data container will effectively become the container
                         # for the newly created node, but transparently to whoever uses
                         # it
-                        
                         self._override()
 
                 if self.can_modify() and self.properties[prop].text != value:
@@ -146,7 +156,7 @@ class SnippetData:
                         oldvalue = self.properties[prop].text
                         self.properties[prop].text = value
                         
-                        if prop == 'tag' or prop == 'accelerator':
+                        if prop == 'tag' or prop == 'accelerator' or prop == 'drop-targets':
                                 container = Library().container(self.language())
                                 container.prop_changed(self, prop, oldvalue)
                 
@@ -160,20 +170,33 @@ class SnippetData:
         
         def is_override(self):
                 return self.override and Library().overridden[self.override]
+        
+        def to_xml(self):
+                return self._create_xml()
 
+        def _create_xml(self, parent=None, update=False, attrib={}):
+                # Create a new node
+                if parent != None:
+                        element = et.SubElement(parent, 'snippet', attrib)
+                else:
+                        element = et.Element('snippet')
+
+                # Create all the properties
+                for p in self.properties:
+                        prop = et.SubElement(element, p)
+                        prop.text = self[p]
+                        
+                        if update:
+                                self.properties[p] = prop
+                
+                return element              
+        
         def _override(self):
                 # Find the user file
                 target = Library().get_user_library(self.language())
 
                 # Create a new node there with override
-                element = et.SubElement(target.root, 'snippet', \
-                                {'override': self.id})
-                
-                # Create all the properties
-                for p in self.properties:
-                        prop = et.SubElement(element, p)
-                        prop.text = self.properties[p]
-                        self.properties[p] = prop
+                element = self._create_xml(target.root, True, {'override': self.id})
 
                 # Create an override snippet data, feed it element so that it stores
                 # all the values and then set the node to None so that it only contains
@@ -247,7 +270,7 @@ class LanguageContainer:
         def __init__(self, language):
                 self.language = language
                 self.snippets = []
-                self.snippets_by_prop = {'tag': {}, 'accelerator': {}}
+                self.snippets_by_prop = {'tag': {}, 'accelerator': {}, 'drop-targets': {}}
                 self.accel_group = gtk.AccelGroup()
                 self._refs = 0
 
@@ -258,8 +281,7 @@ class LanguageContainer:
                 if not value or value == '':
                         return
 
-                snippets_debug('Added ' + prop + ' ' + value + ' to ' + \
-                                str(self.language))
+                snippets_debug('Added ', prop ,' ', value, ' to ', str(self.language))
                 
                 if prop == 'accelerator':
                         keyval, mod = gtk.accelerator_parse(value)
@@ -268,10 +290,14 @@ class LanguageContainer:
                 
                 snippets = self.snippets_by_prop[prop]
                 
-                if value in snippets:
-                        snippets[value].append(snippet)
-                else:
-                        snippets[value] = [snippet]
+                if not isinstance(value, list):
+                        value = [value]
+                
+                for val in value:
+                        if val in snippets:
+                                snippets[val].append(snippet)
+                        else:
+                                snippets[val] = [snippet]
 
         def _remove_prop(self, snippet, prop, value=0):
                 if value == 0:
@@ -280,8 +306,7 @@ class LanguageContainer:
                 if not value or value == '':
                         return
 
-                snippets_debug('Removed ' + prop + ' ' + value + ' from ' + \
-                                str(self.language))
+                snippets_debug('Removed ', prop, ' ', value, ' from ', str(self.language))
 
                 if prop == 'accelerator':
                         keyval, mod = gtk.accelerator_parse(value)
@@ -289,11 +314,15 @@ class LanguageContainer:
 
                 snippets = self.snippets_by_prop[prop]
                 
-                try:
-                        snippets[value].remove(snippet)
-                except:
-                        True
+                if not isinstance(value, list):
+                        value = [value]
                 
+                for val in value:
+                        try:
+                                snippets[val].remove(snippet)
+                        except:
+                                True
+
         def append(self, snippet):
                 tag = snippet['tag']
                 accelerator = snippet['accelerator']
@@ -302,6 +331,7 @@ class LanguageContainer:
                 
                 self._add_prop(snippet, 'tag')
                 self._add_prop(snippet, 'accelerator')
+                self._add_prop(snippet, 'drop-targets')
 
                 return snippet
         
@@ -313,6 +343,7 @@ class LanguageContainer:
                         
                 self._remove_prop(snippet, 'tag')
                 self._remove_prop(snippet, 'accelerator')
+                self._remove_prop(snippet, 'drop-targets')
         
         def prop_changed(self, snippet, prop, oldvalue):
                 snippets_debug('PROP CHANGED (', prop, ')', oldvalue)
@@ -323,10 +354,26 @@ class LanguageContainer:
         def from_prop(self, prop, value):
                 snippets = self.snippets_by_prop[prop]
                 
-                if value in snippets:
-                        return snippets[value]
+                if prop == 'drop-targets':
+                        s = []
+                        
+                        # FIXME: change this to use 
+                        # gnomevfs.mime_type_get_equivalence when it comes
+                        # available
+                        for key, val in snippets.items():
+                                if not value.startswith(key):
+                                        continue
+                                
+                                for snippet in snippets[key]:
+                                        if not snippet in s:
+                                                s.append(snippet)
+                        
+                        return s
                 else:
-                        return []
+                        if value in snippets:
+                                return snippets[value]
+                        else:
+                                return []
         
         def ref(self):
                 self._refs += 1
@@ -353,8 +400,8 @@ class SnippetsSystemFile:
                                 "available, please correct or remove the file.\n")
 
         def _add_snippet(self, element):
-        	if not self.need_id or element.attrib.get('id'):
-                	self.loading_elements.append(element)
+                if not self.need_id or element.attrib.get('id'):
+                        self.loading_elements.append(element)
 
         def set_language(self, element):
                 self.language = element.attrib.get('language')
@@ -386,7 +433,7 @@ class SnippetsSystemFile:
         def _process_element(self, element):
                 if element.tag == 'snippet':
                         self._add_snippet(element)
-                        self.insnippet = False			
+                        self.insnippet = False                        
 
                 return True
 
@@ -581,12 +628,23 @@ class SnippetsUserFile(SnippetsSystemFile):
                 SnippetsSystemFile.unload(self)
                 self.root = None
 
-class LibraryImpl:
-        def __init__(self):
+class Singleton(object):
+        _instance = None
+
+        def __new__(cls, *args, **kwargs):
+                if not cls._instance:
+                        cls._instance = super(Singleton, cls).__new__(
+                                         cls, *args, **kwargs)
+                        cls._instance.__init_once__()
+
+                return cls._instance
+
+class Library(Singleton):        
+        def __init_once__(self):
                 self._accelerator_activated_cb = None
                 self.loaded = False
                 self.check_buffer = gtk.TextBuffer()
-                        
+
         def set_dirs(self, userdir, systemdirs):
                 self.userdir = userdir
                 self.systemdirs = systemdirs
@@ -710,7 +768,6 @@ class LibraryImpl:
         
         def add_override(self, snippet):
                 snippets_debug('Add override:', snippet.override)
-                
                 if not snippet.override in self.overridden:
                         self.overridden[snippet.override] = None
         
@@ -719,7 +776,7 @@ class LibraryImpl:
                 
                 if not library.ok:
                         snippets_debug('Library in wrong format, ignoring')
-                        return
+                        return False
                 
                 snippets_debug('Adding library (' + str(library.language) + '): ' + \
                                 library.path)
@@ -732,6 +789,8 @@ class LibraryImpl:
                                 self.libraries[library.language].append(library)
                 else:
                         self.libraries[library.language] = [library]
+
+                return True
         
         def remove_library(self, library):
                 if not library.ok:
@@ -751,13 +810,13 @@ class LibraryImpl:
                         if snippet.library() == library:
                                 container.remove(snippet)
         
-        def _add_user_library(self, path):
+        def add_user_library(self, path):
                 library = SnippetsUserFile(path)
-                self.add_library(library)
+                return self.add_library(library)
                 
-        def _add_system_library(self, path):
+        def add_system_library(self, path):
                 library = SnippetsSystemFile(path)
-                self.add_library(library)
+                return self.add_library(library)
 
         def find_libraries(self, path, searched, addcb):
                 snippets_debug("Finding in: " + path)
@@ -774,9 +833,6 @@ class LibraryImpl:
                         # Determine what language this file provides snippets for
                         if os.path.isfile(f):
                                 addcb(f)
-                        elif os.path.isdir(f) and not f in searched:
-                                # Recursion rules!
-                                searched = self.find_libraries(self, f, searched, addcb)
                 
                 return searched
         
@@ -798,8 +854,9 @@ class LibraryImpl:
                 
         def get_accel_group(self, language):
                 language = self.normalize_language(language)
-                
                 container = self.container(language)
+
+                self.ensure(language)
                 return container.accel_group
                 
         def save(self, language):
@@ -833,7 +890,7 @@ class LibraryImpl:
                                 self.remove_container(language)
 
         def ensure(self, language):
-        	self.ensure_files()
+                self.ensure_files()
                 language = self.normalize_language(language)
 
                 # Ensure language as well as the global snippets (None)
@@ -851,11 +908,11 @@ class LibraryImpl:
 
                 searched = []
                 searched = self.find_libraries(self.userdir, searched, \
-                                self._add_user_library)
+                                self.add_user_library)
                 
                 for d in self.systemdirs:
                         searched = self.find_libraries(d, searched, \
-                                        self._add_system_library)
+                                        self.add_system_library)
 
                 self.loaded = True
 
@@ -887,6 +944,22 @@ class LibraryImpl:
 
         # Snippet getters
         # ===============
+        def _from_prop(self, prop, value, language=None):
+                self.ensure_files()
+                
+                result = []                
+                language = self.normalize_language(language)
+                        
+                if not language in self.containers:
+                        return []
+
+                self.ensure(language)
+                result = self.containers[language].from_prop(prop, value)
+                
+                if len(result) == 0 and language and None in self.containers:
+                        result = self.containers[None].from_prop(prop, value)
+                
+                return result
         
         # Get snippets for a given language
         def get_snippets(self, language=None):
@@ -903,51 +976,14 @@ class LibraryImpl:
 
         # Get snippets for a given accelerator
         def from_accelerator(self, accelerator, language=None):
-                self.ensure_files()
-                
-                result = []		
-                language = self.normalize_language(language)
-                        
-                if not language in self.containers:
-                        return []
-
-                self.ensure(language)
-                result = self.containers[language].from_prop('accelerator', accelerator)
-                
-                if len(result) == 0 and language and None in self.containers:
-                        result = self.containers[None].from_prop('accelerator', accelerator)
-                
-                return result
+                return self._from_prop('accelerator', accelerator, language)
 
         # Get snippets for a given tag
         def from_tag(self, tag, language=None):
-                self.ensure_files()
-                
-                result = []
-                language = self.normalize_language(language)
-                        
-                if not language in self.containers:
-                        return []
-
-                self.ensure(language)
-                result = self.containers[language].from_prop('tag', tag)
-                
-                if len(result) == 0 and language and None in self.containers:
-                        result = self.containers[None].from_prop('tag', tag)
-                
-                return result
-
-class Library:
-        __instance = None
+                return self._from_prop('tag', tag, language)
         
-        def __init__(self):
-                if not Library.__instance:
-                        Library.__instance = LibraryImpl()
-        
-                self.__dict__['_Library__instance'] = Library.__instance
+        # Get snippets for a given drop target
+        def from_drop_target(self, drop_target, language=None):
+                return self._from_prop('drop-targets', drop_target, language)
                 
-        def __getattr__(self, attr):
-                return getattr(self.__instance, attr)
-        
-        def __setattr__(self, attr, value):
-                return setattr(self.__instance, attr, value)
+# ex:ts=8:et:
